package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.yonge.cooleshow.biz.dal.config.RongCloudConfig;
import com.yonge.cooleshow.biz.dal.dao.ImGroupDao;
import com.yonge.cooleshow.biz.dal.dao.ImGroupMemberDao;
import com.yonge.cooleshow.biz.dal.dao.TeacherDao;
import com.yonge.cooleshow.biz.dal.dto.BasicUserInfo;
import com.yonge.cooleshow.biz.dal.entity.ImGroupMember;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.ImGroupMemberRoleType;
import com.yonge.cooleshow.biz.dal.service.ImGroupMemberService;
import com.yonge.toolset.base.exception.BizException;
import io.rong.models.Result;
import io.rong.models.group.GroupMember;
import io.rong.models.group.GroupModel;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.MessageFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 群成员表(ImGroupMember)表服务实现类
 *
 * @author zx
 * @since 2022-03-22 10:45:58
 */
@Service("imGroupMemberService")
public class ImGroupMemberServiceImpl extends ServiceImpl<ImGroupMemberDao, ImGroupMember> implements ImGroupMemberService {

    private final static Logger log = LoggerFactory.getLogger(ImGroupMemberServiceImpl.class);

    @Resource
    private TeacherDao teacherDao;
    @Resource
    private ImGroupDao imGroupDao;

    @Override
    public ImGroupMemberDao getDao() {
        return this.baseMapper;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertBatch(List<ImGroupMember> imGroupMembers) {
        if(CollectionUtils.isEmpty(imGroupMembers)){
            throw new BizException("群成员列表不可为空");
        }
        this.baseMapper.insertBatch(imGroupMembers);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<GroupMember> initGroupMember(String imGroupId, Long userId, Boolean isAdmin, ImGroupMemberRoleType roleType){
        //记录群成员
        BasicUserInfo basicUserInfo = teacherDao.getBasicUserInfo(userId);
        ImGroupMember imGroupMember = new ImGroupMember(imGroupId,userId,basicUserInfo.getUsername(),basicUserInfo.getAvatar(), isAdmin, roleType);
        Date date = new Date();
        imGroupMember.setCreateTime(date);
        imGroupMember.setUpdateTime(date);
        this.baseMapper.insert(imGroupMember);
        //加入融云群
        List<GroupMember> groupMemberList = new ArrayList<>();

        String imUserId = String.valueOf(userId);
        if (ImGroupMemberRoleType.STUDENT == roleType) {
            imUserId = MessageFormat.format("{0}:{1}", imUserId, ClientEnum.STUDENT.name());
        }

        groupMemberList.add(new GroupMember(imUserId,imGroupId,null));
        return groupMemberList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<GroupMember> initGroupMembers(String imGroupId, Set<Long> userIds, ImGroupMemberRoleType roleType){
        //记录群成员
        List<ImGroupMember> imGroupMembers = baseMapper.initImGroupMember(imGroupId,userIds,roleType.getCode());
        if(CollectionUtils.isNotEmpty(imGroupMembers)){

            for (ImGroupMember item : imGroupMembers) {

                // 用户身份
                item.setRoleType(roleType);
                // 管理员村识
                item.setIsAdmin(ImGroupMemberRoleType.TEACHER == roleType);
            }
            baseMapper.insertBatch(imGroupMembers);

            //加入融云群
            Function<Long,GroupMember> func = (userId) -> {

                String imUserId = String.valueOf(userId);
                if (ImGroupMemberRoleType.STUDENT == roleType) {
                    imUserId = MessageFormat.format("{0}:{1}", imUserId, ClientEnum.STUDENT.name());
                }

                return  new GroupMember(imUserId,imGroupId,null);
            };
            return userIds.stream().map(func).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void join(List<GroupMember> groupMemberList,String imGroupId) throws Exception {
        GroupMember[] groupMembers = groupMemberList.toArray(new GroupMember[groupMemberList.size()]);
        GroupModel groupModel = new GroupModel(imGroupId, 0);
        groupModel.setMembers(groupMembers);
        groupModel.setName(imGroupDao.selectById(imGroupId).getName());
        Result result = RongCloudConfig.rongCloud.group.join(groupModel);
        if(!result.code.equals(200)){
            log.error("加入群组失败:{}",result.errorMessage);
            throw new BizException("加入群组失败：请联系管理员");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delByGroupId(String groupId) {
        this.baseMapper.delByGroupId(groupId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ImGroupMember getUserDetail(String imUserId, String groupId) {

        String[] values = imUserId.split(":");
        // 用户ID
        long userId = Long.parseLong(values[0]);

        ClientEnum clientType = ClientEnum.TEACHER;
        if (values.length > 1 && ClientEnum.STUDENT.match(values[1])) {
            clientType = ClientEnum.STUDENT;
        }

        // 群组信息
        ImGroupMember imGroupMember = baseMapper.findByUserIdAndGroupId(userId, groupId, clientType);

        // 用户基本信息
        BasicUserInfo basicUserInfo = teacherDao.getBasicUserInfo(userId);
        if(Objects.isNull(imGroupMember)){
            imGroupMember = new ImGroupMember();
            imGroupMember.setGroupId(groupId);
            imGroupMember.setUserId(userId);
        }
        //更新当前用户关联的该用户的详情信息
        imGroupMember.setNickname(basicUserInfo.getUsername());
        imGroupMember.setAvatar(basicUserInfo.getAvatar());
        if(imGroupMember.getId() != null){
            imGroupMember.setUpdateTime(new Date());
            baseMapper.updateById(imGroupMember);
        }

        // 设置IM好友ID
        imGroupMember.setImUserId(imUserId);

        return imGroupMember;
    }

    @Transactional(rollbackFor = Exception.class)
    public void startTransactional(Runnable runnable){
        runnable.run();
    }


    /**
     * 群组成员信息
     *
     * @param params Map<String, Object>
     * @return List<ImGroupMember>
     */
    @Override
    public List<ImGroupMember> findChatGroupAllMemberInfo(Map<String, Object> params) {

        Object search = params.get("search");
        List<ImGroupMember> members = getBaseMapper().selectList(Wrappers.<ImGroupMember>query().lambda()
                .and(Objects.nonNull(search) && StringUtils.isNotEmpty(search.toString()),
                        e->e.eq(ImGroupMember::getUserId, search).or()
                                .like(ImGroupMember::getNickname, search))
                .eq(ImGroupMember::getGroupId,params.get("groupId")).orderByDesc(ImGroupMember::getId));

        if (CollectionUtils.isNotEmpty(members)) {

            for (ImGroupMember item : members) {

                item.setImUserId(String.valueOf(item.getUserId()));

                if (ImGroupMemberRoleType.STUDENT == item.getRoleType()) {

                    item.setImUserId(MessageFormat.format("{0}:{1}", String.valueOf(item.getUserId()),
                            ClientEnum.STUDENT.getCode()));
                }
            }
        }

        return Optional.ofNullable(members).orElse(Lists.newArrayList());
    }
}

