package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.yonge.cooleshow.biz.dal.dao.StudentDao;
import com.yonge.cooleshow.biz.dal.entity.Student;
import com.yonge.cooleshow.biz.dal.entity.SysUser;
import com.yonge.cooleshow.biz.dal.entity.TenantActivationCode;
import com.yonge.cooleshow.biz.dal.entity.TenantAlbumPurchase;
import com.yonge.cooleshow.biz.dal.entity.UserTenantAlbumRecord;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.SourceTypeEnum;
import com.yonge.cooleshow.biz.dal.mapper.SysUserMapper;
import com.yonge.cooleshow.biz.dal.mapper.TenantActivationCodeMapper;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumPurchaseMapper;
import com.yonge.cooleshow.biz.dal.mapper.UserTenantAlbumRecordMapper;
import com.yonge.cooleshow.biz.dal.service.TenantActivationCodeService;
import com.yonge.cooleshow.biz.dal.service.TenantAlbumService;
import com.yonge.cooleshow.biz.dal.vo.UserOrderDetailVo;
import com.yonge.cooleshow.biz.dal.wrapper.TenantActivationCodeWrapper;
import com.yonge.cooleshow.biz.dal.wrapper.TenantAlbumWrapper;
import com.yonge.cooleshow.common.enums.EActivationCode;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.utils.easyexcel.ExcelDataReaderProperty;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 机构激活码
 * 2023-07-21 17:32:49
 */
@Slf4j
@Service
public class TenantActivationCodeServiceImpl extends ServiceImpl<TenantActivationCodeMapper, TenantActivationCode> implements TenantActivationCodeService {

    @Autowired
    private StudentDao studentDao;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private TenantAlbumPurchaseMapper tenantAlbumPurchaseMapper;

    @Autowired
    private UserTenantAlbumRecordMapper userTenantAlbumRecordMapper;

    @Autowired
    private TenantAlbumService tenantAlbumService;

    /**
     * 查询详情
     *
     * @param id 详情ID
     * @return TenantActivationCode
     */
    @Override
    public TenantActivationCode detail(Long id) {

        return baseMapper.selectById(id);
    }

    /**
     * 分页查询
     *
     * @param page  IPage<TenantActivationCode>
     * @param query TenantActivationCodeWrapper.TenantActivationCodeQuery
     * @return IPage<TenantActivationCode>
     */
    @Override
    public IPage<TenantActivationCodeWrapper.TenantActivationCode> selectPage(IPage<TenantActivationCodeWrapper.TenantActivationCode> page,
                                                                              TenantActivationCodeWrapper.TenantActivationCodeQuery query) {

        return page.setRecords(baseMapper.selectPage(page, query));
    }

    /**
     * 添加
     *
     * @param tenantActivationCode TenantActivationCodeWrapper.TenantActivationCode
     * @return Boolean
     */
    @Override
    public Boolean add(TenantActivationCodeWrapper.TenantActivationCode tenantActivationCode) {

        return this.save(JSON.parseObject(tenantActivationCode.jsonString(), TenantActivationCode.class));
    }

    /**
     * 更新
     *
     * @param tenantActivationCode TenantActivationCodeWrapper.TenantActivationCode
     * @return Boolean
     */
    @Override
    public Boolean update(TenantActivationCodeWrapper.TenantActivationCode tenantActivationCode) {

        return this.updateById(JSON.parseObject(tenantActivationCode.jsonString(), TenantActivationCode.class));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void sendActivationCode(Long tenantId, Long tenantAlbumPurchaseId, List<String> activationCodeList,
                                   List<Long> studentIdList) {
        QueryWrapper<TenantActivationCode> queryWrapper = new QueryWrapper<>();
        if (CollectionUtils.isEmpty(activationCodeList)) {
            // 入参为空走批量发放方式
            queryWrapper.lambda().eq(TenantActivationCode::getTenantId, tenantId)
                    .eq(TenantActivationCode::getTenantAlbumPurchaseId, tenantAlbumPurchaseId)
                    .eq(TenantActivationCode::getSendStatus, EActivationCode.WAIT)
                    .eq(TenantActivationCode::getActivationStatus, false);
        } else {
            queryWrapper.lambda().eq(TenantActivationCode::getTenantId, tenantId)
                    .eq(TenantActivationCode::getTenantAlbumPurchaseId, tenantAlbumPurchaseId)
                    .in(TenantActivationCode::getActivationCode, activationCodeList)
                    .eq(TenantActivationCode::getSendStatus, EActivationCode.WAIT)
                    .eq(TenantActivationCode::getActivationStatus, false);
        }

        List<TenantActivationCode> tenantActivationCodes = baseMapper.selectList(queryWrapper);
        if (tenantActivationCodes.isEmpty()) {
            throw new BizException("无可发放的激活码");
        }

        QueryWrapper<Student> studentQueryWrapper = new QueryWrapper<>();
        studentQueryWrapper.lambda().eq(Student::getTenantId, tenantId)
                .in(Student::getUserId, studentIdList);
        List<Student> students = studentDao.selectList(studentQueryWrapper);
        if (tenantActivationCodes.size() < students.size()) {
            throw new BizException("学生的数量不能大于可用的激活码数量");
        }

        List<Long> studentIds = students.stream().map(Student::getUserId).collect(Collectors.toList());
        Map<Long, String> idPhoneMap = sysUserMapper.selectBatchIds(studentIds).stream()
                .collect(Collectors.toMap(SysUser::getId, SysUser::getPhone));

        for (Long studentId : studentIds) {
            for (TenantActivationCode tenantActivationCode : tenantActivationCodes) {
                boolean update = this.lambdaUpdate()
                        .set(TenantActivationCode::getSendStatus, EActivationCode.SEND.getCode())
                        .set(TenantActivationCode::getActivationPhone, idPhoneMap.get(studentId))
                        .eq(TenantActivationCode::getActivationStatus, false)
                        .eq(TenantActivationCode::getId, tenantActivationCode.getId())
                        .update();
                if (!update) {
                    throw new BizException("激活码已被激活:" + tenantActivationCode.getActivationCode());
                }
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void active(String activationCode, Long studentId) {
        Student student = studentDao.selectById(studentId);
        if (student == null) {
            throw new BizException("学生不存在");
        }
        Long tenantId = -1L;
        if (student.getTenantId() != null) {
            tenantId = student.getTenantId();
        }

        TenantActivationCode code = this.lambdaQuery().eq(TenantActivationCode::getTenantId, tenantId)
                .eq(TenantActivationCode::getActivationCode, activationCode)
                .last("limit 1").one();
        if (code == null) {
            throw new BizException("激活码不存在");
        }
        SysUser sysUser = sysUserMapper.selectById(studentId);
        if (Boolean.TRUE.equals(code.getActivationStatus()) || (EActivationCode.SEND.equals(code.getSendStatus()) &&
                !sysUser.getPhone().equals(code.getActivationPhone()))) {
            throw new BizException("激活码已经被使用");
        }

        // 通过状态和ID同时判断更新是否存在竞争
        boolean update = this.lambdaUpdate()
                .set(TenantActivationCode::getActivationStatus, true)
                .set(TenantActivationCode::getActivationUserId, student.getUserId())
                .set(TenantActivationCode::getActivationTime, new Date())
                .set(TenantActivationCode::getActivationPhone, sysUser.getPhone())
                .eq(TenantActivationCode::getId, code.getId())
                .eq(TenantActivationCode::getActivationStatus, false)
                .update();
        if (!update) {
            throw new BizException("激活码已经被使用");
        }
        Long tenantAlbumPurchaseId = code.getTenantAlbumPurchaseId();
        TenantAlbumPurchase purchase = tenantAlbumPurchaseMapper.selectById(tenantAlbumPurchaseId);

        addUserTenantAlbumRecord(student.getUserId(), purchase, null);

        // 更新购买记录中激活码使用统计数量值
        Integer activeCodeNumber = this.lambdaQuery()
                .eq(TenantActivationCode::getTenantId, tenantId)
                .eq(TenantActivationCode::getTenantAlbumPurchaseId, code.getTenantAlbumPurchaseId())
                .eq(TenantActivationCode::getActivationStatus, true).count();

        TenantAlbumPurchase tenantAlbumPurchase = new TenantAlbumPurchase();
        tenantAlbumPurchase.setId(code.getTenantAlbumPurchaseId());
        tenantAlbumPurchase.setActiveQuantity(activeCodeNumber);
        tenantAlbumPurchaseMapper.updateById(tenantAlbumPurchase);
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void activeById(String id, Long userId) {
        Student student = studentDao.selectById(userId);
        if (student == null) {
            throw new BizException("学生不存在");
        }
        Long tenantId = -1L;
        if (student.getTenantId() != null) {
            tenantId = student.getTenantId();
        }
        TenantActivationCode code = baseMapper.selectById(id);
        if(code == null || !code.getTenantId().equals(tenantId)) {
            throw new BizException("激活码不存在");
        }
        SysUser sysUser = sysUserMapper.selectById(student.getUserId());
        if(!sysUser.getPhone().equals(code.getActivationPhone())) {
            // 该激活码未指定为当前学生使用
            throw new BizException("激活码不存在");
        }
        if (Boolean.TRUE.equals(code.getActivationStatus())) {
            throw new BizException("激活码已经被使用");
        }

        // 通过状态和ID同时判断更新是否存在竞争
        boolean update = this.lambdaUpdate()
                .set(TenantActivationCode::getActivationStatus, true)
                .set(TenantActivationCode::getActivationUserId, student.getUserId())
                .set(TenantActivationCode::getActivationTime, new Date())
                .set(TenantActivationCode::getActivationPhone, sysUser.getPhone())
                .eq(TenantActivationCode::getId, code.getId())
                .eq(TenantActivationCode::getActivationStatus, false)
                .update();
        if (!update) {
            throw new BizException("激活码已经被使用");
        }
        Long tenantAlbumPurchaseId = code.getTenantAlbumPurchaseId();
        TenantAlbumPurchase purchase = tenantAlbumPurchaseMapper.selectById(tenantAlbumPurchaseId);

        addUserTenantAlbumRecord(student.getUserId(), purchase, null);

        // 更新购买记录中激活码使用统计数量值
        Integer activeCodeNumber = this.lambdaQuery()
                .eq(TenantActivationCode::getTenantId, tenantId)
                .eq(TenantActivationCode::getTenantAlbumPurchaseId, code.getTenantAlbumPurchaseId())
                .eq(TenantActivationCode::getActivationStatus, true).count();

        TenantAlbumPurchase tenantAlbumPurchase = new TenantAlbumPurchase();
        tenantAlbumPurchase.setId(code.getTenantAlbumPurchaseId());
        tenantAlbumPurchase.setActiveQuantity(activeCodeNumber);
        tenantAlbumPurchaseMapper.updateById(tenantAlbumPurchase);

    }


    /**
     * 添加用户机构专辑激活记录
     *
     * @param studentId         学生ID
     * @param userOrderDetailVo 订单详情
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addUserTenantAlbumRecord(Long studentId, UserOrderDetailVo userOrderDetailVo) {
        addUserTenantAlbumRecord(studentId, null, userOrderDetailVo);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void importActiveCode(List<ExcelDataReaderProperty<TenantActivationCodeWrapper.ImportTemplate>> dataList,
                                 Long tenantId, Long userId, Long tenantAlbumPurchaseId) {
        if (dataList.isEmpty()) {
            return;
        }
        dataList.sort(Comparator.comparingInt(ExcelDataReaderProperty::getRowIndex));

        List<String> errMsg = new ArrayList<>();
        Map<String, Integer> codeRowMap = new HashMap<>();
        Map<String, String> codePhoneMap = new HashMap<>();

        String errTemplate = "第%s行%s";
        // 校验数据格式是否错误
        for (ExcelDataReaderProperty<TenantActivationCodeWrapper.ImportTemplate> next : dataList) {
            Integer rowIndex = next.getRowIndex();
            TenantActivationCodeWrapper.ImportTemplate code = next.getClazz();

            int msgRowNo = rowIndex + 1;
            code.checkIsIllegal().forEach(err -> errMsg.add(String.format(errTemplate, msgRowNo, err)));
            if (codeRowMap.containsKey(code.getCode())) {
                errMsg.add(String.format(errTemplate, msgRowNo, "激活码重复"));
            }
            codeRowMap.put(code.getCode().trim(), msgRowNo);
            if (StringUtils.isNotEmpty(code.getPhone())) {
                codePhoneMap.put(code.getCode().trim(), code.getPhone().trim());
            }

            if (errMsg.size() > 100) {
                break;
            }
        }

        if (codePhoneMap.isEmpty()) {
            throw new BizException("请指定手机号码激活");
        }

        if (!errMsg.isEmpty()) {
            throw new BizException(String.join(",", errMsg));
        }
        // 校验激活码是否被使用
        List<List<String>> codePartition = Lists.partition(new ArrayList<>(codeRowMap.keySet()), 50);

        List<TenantActivationCode> tenantActivationCodes = new ArrayList<>();
        for (List<String> codes : codePartition) {
            List<TenantActivationCode> activationCodes = this.lambdaQuery()
                    .eq(TenantActivationCode::getTenantId, tenantId)
                    .eq(TenantActivationCode::getTenantAlbumPurchaseId, tenantAlbumPurchaseId)
                    .in(TenantActivationCode::getActivationCode, codes).list();

            // 存在无效码或者已经使用过的码
            if (codes.size() != activationCodes.size() ||
                    activationCodes.stream().anyMatch(TenantActivationCode::getActivationStatus)) {
                Map<String, Boolean> codeStatusMap = activationCodes.stream()
                        .collect(Collectors.toMap(TenantActivationCode::getActivationCode,
                                TenantActivationCode::getActivationStatus));
                for (String code : codes) {
                    if (!codeStatusMap.containsKey(code)) {
                        errMsg.add(String.format(errTemplate, codeRowMap.get(code), "验证码无效"));
                    } else if (Boolean.TRUE.equals(codeStatusMap.get(code))) {
                        errMsg.add(String.format(errTemplate, codeRowMap.get(code), "验证码已经激活"));
                    }
                }
            }

            List<TenantActivationCode> updateDataList = activationCodes.stream().map(next -> {
                TenantActivationCode tenantActivationCode = new TenantActivationCode();
                tenantActivationCode.setId(next.getId());
                tenantActivationCode.setActivationPhone(codePhoneMap.get(next.getActivationCode()));
                tenantActivationCode.setSendStatus(EActivationCode.SEND);
                return tenantActivationCode;
            }).collect(Collectors.toList());

            tenantActivationCodes.addAll(updateDataList);
        }
        if (!errMsg.isEmpty()) {
            throw new BizException(String.join(",", errMsg));
        }
        if (!tenantActivationCodes.isEmpty()) {
            this.updateBatchById(tenantActivationCodes, 50);
        }
    }

    private void addUserTenantAlbumRecord(Long studentId, TenantAlbumPurchase purchase,
                                          UserOrderDetailVo userOrderDetailVo) {


        UserTenantAlbumRecord userTenantAlbumRecord = new UserTenantAlbumRecord();
        if (userOrderDetailVo != null) {

            TenantAlbumWrapper.TenantAlbumContent tenantAlbumContent = JSON
                    .parseObject(userOrderDetailVo.getBizContent(), TenantAlbumWrapper.TenantAlbumContent.class);

            userTenantAlbumRecord.setTenantId(tenantAlbumContent.getTenantId());
            userTenantAlbumRecord.setUserId(studentId);
            userTenantAlbumRecord.setTenantAlbumId(tenantAlbumContent.getTenantAlbumId());
            userTenantAlbumRecord.setOrderNo(userOrderDetailVo.getOrderNo());
            userTenantAlbumRecord.setSourceType(SourceTypeEnum.ORDER);
            userTenantAlbumRecord.setClientType(ClientEnum.STUDENT);
            userTenantAlbumRecord.setSubOrderNo(userOrderDetailVo.getSubOrderNo());
            userTenantAlbumRecord.setCreateBy(studentId);
            userTenantAlbumRecord.setTimes(tenantAlbumContent.getBuyCycle());
        } else if (purchase != null) {

            userTenantAlbumRecord.setTenantId(purchase.getTenantId());
            userTenantAlbumRecord.setUserId(studentId);
            userTenantAlbumRecord.setTenantAlbumId(purchase.getTenantAlbumId());
            userTenantAlbumRecord.setOrderNo(purchase.getId().toString());
            userTenantAlbumRecord.setSourceType(SourceTypeEnum.TENANT);
            userTenantAlbumRecord.setClientType(ClientEnum.STUDENT);
            userTenantAlbumRecord.setSubOrderNo(purchase.getId().toString());
            userTenantAlbumRecord.setCreateBy(studentId);
            userTenantAlbumRecord.setTimes(purchase.getPurchaseCycle());
        }

        List<UserTenantAlbumRecord> userTenantAlbumRecords = userTenantAlbumRecordMapper
                .selectList(Wrappers.<UserTenantAlbumRecord>lambdaQuery()
                        .eq(UserTenantAlbumRecord::getTenantId, userTenantAlbumRecord.getTenantId())
                        .eq(UserTenantAlbumRecord::getUserId, studentId)
                        .eq(UserTenantAlbumRecord::getTenantAlbumId, userTenantAlbumRecord.getTenantAlbumId())
                        .eq(UserTenantAlbumRecord::getClientType, ClientEnum.STUDENT)
                        .orderByDesc(UserTenantAlbumRecord::getEndTime));

        userTenantAlbumRecord.setUserId(studentId);
        userTenantAlbumRecord.setCreateBy(studentId);


        userTenantAlbumRecord.setType("MONTH");

        Calendar instance = Calendar.getInstance();
        if (userTenantAlbumRecords.isEmpty()) {
            instance.setTime(DateTime.now().withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0).toDate());
        } else {
            // 如果最后一次的时间的小于当前时间，则以当前时间为会员的开始时间
            // 如果最后一次的时间的大于当前时间，则以最后一次的结束时间为记录的开始时间，相当会员续期
            UserTenantAlbumRecord lastRecord = userTenantAlbumRecords.get(0);
            Date lastEndTime = lastRecord.getEndTime();
            if (lastEndTime.before(new Date())) {
                instance.setTime(DateTime.now().withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0).toDate());
            } else {
                instance.setTime(lastEndTime);
            }
        }
        userTenantAlbumRecord.setStartTime(instance.getTime());
        instance.add(Calendar.MONTH, userTenantAlbumRecord.getTimes());

        instance.set(Calendar.HOUR_OF_DAY, 23);
        instance.set(Calendar.MINUTE, 59);
        instance.set(Calendar.SECOND, 59);
        instance.set(Calendar.MILLISECOND, 0);
        userTenantAlbumRecord.setEndTime(instance.getTime());
        userTenantAlbumRecordMapper.insert(userTenantAlbumRecord);
    }
}