package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.beans.BeanUtils;
import lombok.extern.slf4j.Slf4j;
import com.yonge.cooleshow.biz.dal.entity.TenantAlbumPurchase;
import com.yonge.cooleshow.biz.dal.wrapper.TenantAlbumPurchaseWrapper;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumPurchaseMapper;
import com.yonge.cooleshow.biz.dal.service.TenantAlbumPurchaseService;

import java.util.List;

/**
 * 机构专辑采购
 * 2023-07-21 17:32:49
 */
@Slf4j
@Service
public class TenantAlbumPurchaseServiceImpl extends ServiceImpl<TenantAlbumPurchaseMapper, TenantAlbumPurchase> implements TenantAlbumPurchaseService {

    /**
     * 查询详情
     *
     * @param id 详情ID
     * @return TenantAlbumPurchase
     */
    @Override
    public TenantAlbumPurchase detail(Long id) {

        return baseMapper.selectById(id);
    }

    /**
     * 分页查询
     *
     * @param page  IPage<TenantAlbumPurchase>
     * @param query TenantAlbumPurchaseWrapper.TenantAlbumPurchaseQuery
     * @return IPage<TenantAlbumPurchase>
     */
    @Override
    public IPage<TenantAlbumPurchaseWrapper.TenantAlbumPurchase> selectPage(IPage<TenantAlbumPurchaseWrapper.TenantAlbumPurchase> page,
                                                                            TenantAlbumPurchaseWrapper.TenantAlbumPurchaseQuery query) {

        List<TenantAlbumPurchaseWrapper.TenantAlbumPurchase> selectPage = baseMapper.selectPage(page, query);
        return page.setRecords(selectPage);
    }

    /**
     * 添加
     *
     * @param tenantAlbumPurchase TenantAlbumPurchaseWrapper.TenantAlbumPurchase
     * @return Boolean
     */
    @Override
    public Boolean add(TenantAlbumPurchaseWrapper.TenantAlbumPurchase tenantAlbumPurchase) {

        return this.save(JSON.parseObject(tenantAlbumPurchase.jsonString(), TenantAlbumPurchase.class));
    }

    /**
     * 更新
     *
     * @param tenantAlbumPurchase TenantAlbumPurchaseWrapper.TenantAlbumPurchase
     * @return Boolean
     */
    @Override
    public Boolean update(TenantAlbumPurchaseWrapper.TenantAlbumPurchase tenantAlbumPurchase) {

        return this.updateById(JSON.parseObject(tenantAlbumPurchase.jsonString(), TenantAlbumPurchase.class));
    }

    @Override
    public TenantAlbumPurchase getByOrderNo(String orderNo) {
        return this.lambdaQuery()
                .eq(TenantAlbumPurchase::getOrderNo, orderNo)
                .last("limit 1")
                .one();

    }
}