package com.yonge.cooleshow.tenant.controller;

import com.alibaba.fastjson.JSON;
import com.google.gson.JsonObject;
import com.timevale.tech.sdk.seal.IFontsLoader;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.entity.TenantAlbumPurchase;
import com.yonge.cooleshow.biz.dal.entity.TenantInfo;
import com.yonge.cooleshow.biz.dal.entity.TenantStaff;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.wrapper.TenantInfoWrapper;
import com.yonge.cooleshow.common.constant.SysConfigConstant;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.security.SecurityConstants;
import com.yonge.cooleshow.tenant.io.request.SysUserWrapper;
import com.yonge.cooleshow.tenant.io.request.TenantInfoVo;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.utils.obj.ObjectUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanInstantiationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * @Author：haonan
 * @Date：2023/7/24 9:57
 * @Filename：TenantInfoController
 */
@RestController
@RequestMapping("/tenantInfoUpdate")
@Api(value = "机构信息修改", tags = "机构信息修改")
public class TenantInfoController extends BaseController {

    @Autowired
    TenantInfoService tenantInfoService;

    @Autowired
    SysUserFeignService sysUserFeignService;

    @Autowired
    private SmsCodeService smsCodeService;

    @Autowired
    private TenantStaffService tenantStaffService;

    @Autowired
    private SysConfigService sysConfigService;

    @ApiOperation(value = "发送登录短信验证码")
    @ApiImplicitParams({@ApiImplicitParam(name = "mobile", value = "手机号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "类型（PASSWD:修改密码，LOGIN:登录或注册,BANK:绑定银行卡,PHONE:修改手机号）", required =
                    true, dataType = "String")})
    @PostMapping(value = "/sendSmsCode")
    public Object sendLoginVerifyCode(String mobile, String type) throws Exception {
        smsCodeService.sendValidCode(mobile, type, ClientEnum.SYSTEM);
        return succeed();
    }

    @ApiOperation(value = "校验短信验证码")
    @ApiImplicitParams({@ApiImplicitParam(name = "phone", value = "手机号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "code", value = "短信验证码", required = true, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "类型（PASSWD:修改密码，LOGIN:登录或注册,BANK:绑定银行卡,PHONE:修改手机号）", required =
                    true, dataType = "String")})
    @PostMapping(value = "/verifySmsCode")
    public Object verifySmsCode(String phone, String code, String type) {
        if (StringUtils.isEmpty(phone) || StringUtils.isEmpty(code)) {
            return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
        }
        if (smsCodeService.verifyValidCode(phone, code, type)) {
            return succeed();
        }
        return failed("验证码校验失败");
    }


    /**
     * 机构修改数据
     */
    @PostMapping("/update")
    @ApiOperation(value = "修改机构信息", notes = "传入TenantInfo")
//    @PreAuthorize("@pcs.hasPermissions('tenantInfoUpdate/update')")
    public HttpResponseResult<Boolean> updateTenantInfo(@Valid @RequestBody TenantInfoVo.TenantInfo info) {

        TenantInfo tenantInfo = JSON.parseObject(info.jsonString(), TenantInfo.class);
            tenantInfoService.updateById(tenantInfo);

        return  HttpResponseResult.succeed(true);

    }




    /**
     * 当前机构查询详细
     */
    @PostMapping("/detail")
    @ApiOperation(value = "查询详细", notes = "查询详细")
//    @PreAuthorize("@pcs.hasPermissions('tenantInfoUpdate/detail')")
    public HttpResponseResult<TenantInfoWrapper.TenantInfo> detail() {
        TenantInfoWrapper.TenantInfo info = new TenantInfoWrapper.TenantInfo();
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Long id = sysUser.getId();
        List<TenantStaff> list = tenantStaffService.lambdaQuery().eq(TenantStaff::getUserId, id).list();
        if (CollectionUtils.isNotEmpty(list)){
            TenantStaff staff = list.get(0);
            Long tenantId = staff.getTenantId();
            if (!(tenantId == null)){
                info = tenantInfoService.detailTenantInfo(tenantId);
            }
        }
        return succeed(info);
    }



    /**
     * 当前用户查询详细
     */
    @PostMapping("/sysUserDetail")
    @ApiOperation(value = "查询详细", notes = "查询详细")
//    @PreAuthorize("@pcs.hasPermissions('tenantInfoUpdate/sysUserDetail')")
    public HttpResponseResult<SysUser> sysUserDetail() {

        SysUser sysUser = sysUserFeignService.queryUserInfo();


        return HttpResponseResult.succeed(sysUser);
    }

    /**
     * 当前用户修改信息
     */
    @PostMapping("/sysUserUpdate")
    @ApiOperation(value = "用户修改", notes = "用户修改")
//    @PreAuthorize("@pcs.hasPermissions('tenantInfoUpdate/sysUserUpdate')")
    public HttpResponseResult<Boolean> sysUserUpdate(@RequestBody SysUserWrapper.SysUser sysUser) {
        String code = sysUser.getCode();
        String phone = sysUser.getPhone();

        com.yonge.cooleshow.biz.dal.entity.SysUser sysUser1 = JSON.parseObject(sysUser.jsonString(), com.yonge.cooleshow.biz.dal.entity.SysUser.class);

        //如果传手机号
        if (StringUtils.isNotBlank(phone)){
            boolean re = smsCodeService.verifyValidCode(phone, code, "PHONE");
            if (re){
                SysUser user = sysUserFeignService.queryUserInfo();
                tenantInfoService.updateSysUser(sysUser1, user.getId());

                //设置默认头像
                if (StringUtils.isEmpty(sysUser.getAvatar())) {
                    sysUser.setAvatar(sysConfigService.findConfigValue(SysConfigConstant.DEFAULT_HEARD));
                }

                //判断更改的手机所属的机构信息

                String oldPhone = user.getPhone();
                //查看原机构信息
                List<TenantInfo> list = tenantInfoService.lambdaQuery().eq(TenantInfo::getPhone, oldPhone).list();
                if (CollectionUtils.isNotEmpty(list)){
                    TenantInfo info = list.get(0);
                    if (ObjectUtil.isEmpty(info)) {
                        //更新tennatInfo
                        info.setPhone(phone);
                        tenantInfoService.updateById(info);
                    }
                }

                //更新员工表
                Long id = user.getId();
                TenantStaff staff = tenantStaffService.getByUserId(id);
                if (staff != null){
                    staff.setNickname(sysUser.getUsername());
                    staff.setAvatar(sysUser.getAvatar());
                    tenantStaffService.updateById(staff);
                }
            } else {
                throw new BizException("手机号校验有误");
            }
        }  else {
            SysUser user = sysUserFeignService.queryUserInfo();
            tenantInfoService.updateSysUser(sysUser1, user.getId());

            //设置默认头像
            if (StringUtils.isBlank(sysUser.getAvatar())) {
                sysUser.setAvatar(sysConfigService.findConfigValue(SysConfigConstant.DEFAULT_HEARD));
            }

            //更新员工表
            Long id = user.getId();
            TenantStaff staff = tenantStaffService.getByUserId(id);
            if (staff != null){
                staff.setNickname(sysUser.getUsername());
                staff.setAvatar(sysUser.getAvatar());
                tenantStaffService.updateById(staff);
            }
        }
            return HttpResponseResult.succeed(true);
        }
}
